package com.io.yutian.aulib.command.list;

import com.io.yutian.aulib.command.CommandContext;
import com.io.yutian.aulib.command.CommandNode;
import com.io.yutian.aulib.command.ICommand;
import com.io.yutian.aulib.command.ICommandManager;
import com.io.yutian.aulib.command.argument.Argument;
import com.io.yutian.aulib.command.argument.ArgumentTypes;
import com.io.yutian.aulib.lang.Lang;
import com.io.yutian.aulib.util.ComponentBuilder;
import com.io.yutian.aulib.list.PageList;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CommandHelp extends ICommand {

    private ICommandManager commandManager;
    @Nullable
    private String alias;

    public CommandHelp(ICommandManager commandManager) {
        this(commandManager, null);
    }

    public CommandHelp(ICommandManager commandManager, String alias) {
        super("help");
        this.commandManager = commandManager;
        this.alias = alias;
        addArgument(Argument.argument("page", ArgumentTypes.INTEGER).optional(1));
    }

    @Override
    public boolean hasPermission(CommandSender sender) {
        return true;
    }

    @Override
    public void executes(CommandContext commandContext) {
        String commandAlias = alias != null ? alias : commandContext.getLabel();
        CommandSender sender = commandContext.getSender();
        int page = commandContext.getArgumentsValue("page").getInt();
        if (page <= 0) {
            sender.sendMessage(Lang.get("command.help.page-error"));
            return;
        }
        List<ICommand> commands = commandManager.getCommands();
        Stream<ICommand> stream = commands.stream().filter((c) -> c.hasPermission(sender));
        List<ICommand> list = stream.collect(Collectors.toList());
        PageList<ICommand> pageList = new PageList<>(list, 8);
        if (page > pageList.size()) {
            sender.sendMessage(Lang.get("command.help.page-error"));
            return;
        }
        sender.sendMessage(" ");
        List<ICommand> commandList = pageList.getList(page);
        sender.sendMessage("§7======[ §e§l"+commandManager.getName()+" §7]======");
        for (ICommand command : commandList) {
            StringBuilder stringBuilder = new StringBuilder("§6/"+commandAlias+" "+command.getName());
            stringBuilder.append("§f");
            if (command.getCommandNodes().size() > 0) {
                StringBuilder sb = new StringBuilder();
                sb.append(" [");
                int i = 0;
                for (CommandNode node : command.getCommandNodes()) {
                    sb.append(node.getName());
                    if (i + 1 < command.getCommandNodes().size()) {
                        sb.append("/");
                    }
                    i++;
                }
                sb.append("]");
                stringBuilder.append(sb);
            } else {
                for (Argument argument : command.getArguments()) {
                    stringBuilder.append(" ");
                    stringBuilder.append("<"+argument.getName()+">");
                }
            }
            if (command.getDescription() != null) {
                stringBuilder.append(" ");
                stringBuilder.append("§7- §f"+command.getDescription());
            }
            sender.sendMessage(stringBuilder.toString());
        }
        ComponentBuilder componentBuilder = new ComponentBuilder();
        boolean hasUpPage = page > 1;
        boolean hasNextPage = page < pageList.size();
        componentBuilder.add("§7=====");
        if (hasUpPage) {
            componentBuilder.add(" §7["+getColor(true)+"◀§7] ", ClickEvent.clickEvent(ClickEvent.Action.RUN_COMMAND, "/"+commandAlias+" help "+(page-1)), HoverEvent.hoverEvent(HoverEvent.Action.SHOW_TEXT, Component.text("§f上一页")));
        } else {
            componentBuilder.add(" §7["+getColor(false)+"◀§7] ");
        }
        componentBuilder.add("§7====");
        componentBuilder.add("(§a"+page+"§f/§e"+pageList.size()+"§7)");
        componentBuilder.add("§7====");
        if (hasNextPage) {
            componentBuilder.add(" §7["+getColor(true)+"▶§7] ", ClickEvent.clickEvent(ClickEvent.Action.RUN_COMMAND, "/"+commandAlias+" help "+(page+1)), HoverEvent.hoverEvent(HoverEvent.Action.SHOW_TEXT, Component.text("§f下一页")));
        } else {
            componentBuilder.add(" §7["+getColor(false)+"▶§7] ");
        }
        componentBuilder.add("§7=====");
        sender.sendMessage(componentBuilder.build());
    }

    private String getColor(boolean hasPage) {
        return hasPage ? "§a" : "§c";
    }

}