package com.io.yutian.aulib.util;

import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;
import org.bukkit.*;
import org.bukkit.block.banner.Pattern;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.EnchantmentStorageMeta;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.SkullMeta;
import org.bukkit.potion.PotionEffect;

import java.lang.reflect.Field;
import java.util.*;

public class ItemStackBuilder {

    private ItemStack itemStack;

    private Material type;

    private int amount;

    private short data;

    private ItemMeta meta;

    private String displayName;

    private int customModelData;

    private List<String> lore;

    private Map<Enchantment, Integer> enchants;

    private Set<ItemFlag> itemFlags;

    private List<PotionEffect> potionEffects;
    private List<Pattern> patterns;
    private List<FireworkEffect> fireworkEffects;
    private String skullTextures;
    private UUID skullOwner;

    public ItemStackBuilder(Material material) {
        this.itemStack = new ItemStack(material);
        this.type = material;
        this.amount = 1;
        this.data = 0;
        this.meta = this.itemStack.getItemMeta();
        this.displayName = this.meta.getDisplayName();
        this.customModelData = this.meta.hasCustomModelData() ? this.meta.getCustomModelData() : -1;
        this.lore = new ArrayList<>();
        this.enchants = new HashMap<>();
        this.itemFlags = new HashSet<>();
        this.potionEffects = new ArrayList<>();
        this.patterns = new ArrayList<>();
        this.fireworkEffects = new ArrayList<>();
    }

    public ItemStackBuilder(ItemStack item) {
        this.itemStack = item.clone();
        this.type = item.getType();
        this.amount = item.getAmount();
        this.data = item.getDurability();
        this.meta = item.getItemMeta();
        this.displayName = this.meta.getDisplayName();
        this.customModelData = this.meta.hasCustomModelData() ? this.meta.getCustomModelData() : -1;
        this.lore = this.meta.getLore() == null ? new ArrayList<>() : meta.getLore();
        this.enchants = this.meta.getEnchants();
        this.itemFlags = this.meta.getItemFlags();
        this.potionEffects = new ArrayList<>();
        this.patterns = new ArrayList<>();
        this.fireworkEffects = new ArrayList<>();
    }

    public ItemStack build() {
        ItemStack item = new ItemStack(type, amount);
        item.setType(type);
        item.setAmount(amount);
        ItemMeta itemMeta = item.getItemMeta();
        if (displayName != null) {
            itemMeta.setDisplayName(ChatColor.translateAlternateColorCodes('&', displayName));
        }
        if (enchants != null) {
            item.addUnsafeEnchantments(enchants);
        }
        if (lore != null) {
            List<String> l = new ArrayList<>();
            lore.forEach((s) -> {
                l.add(ChatColor.translateAlternateColorCodes('&', s));
            });
            itemMeta.setLore(l);
        }
        if (this.customModelData != -1) {
            itemMeta.setCustomModelData(customModelData);
        }
        if (itemFlags != null) {
            for (ItemFlag flag : itemFlags) {
                itemMeta.addItemFlags(flag);
            }
        }
        if (fireworkEffects != null && fireworkEffects.size() > 0 && (type.equals(Material.FIREWORK_STAR) || type.equals(Material.FIREWORK_ROCKET))) {
            FireworkMeta fireworkMeta = (FireworkMeta) itemMeta;
            for (FireworkEffect effect : fireworkEffects) {
                fireworkMeta.addEffect(effect);
            }
        }
        if (skullTextures != null && type.equals(Material.PLAYER_HEAD)) {
            SkullMeta skullMeta = (SkullMeta) itemMeta;
            String uuid = UUID.randomUUID().toString();

            GameProfile profile = new GameProfile(UUID.fromString(uuid), null);
            profile.getProperties().put("textures", new Property("textures", skullTextures));
            Field profileField;
            try {
                profileField = skullMeta.getClass().getDeclaredField("profile");
                profileField.setAccessible(true);
                profileField.set(skullMeta, profile);
            } catch (NullPointerException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e1) {
            }
        } else if (skullOwner != null && type.equals(Material.PLAYER_HEAD)) {
            SkullMeta skullMeta = (SkullMeta) itemMeta;
            OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(skullOwner);
            if (offlinePlayer != null) {
                skullMeta.setOwningPlayer(Bukkit.getOfflinePlayer(skullOwner));
            }
        }
        item.setItemMeta(itemMeta);
        return item;
    }

    public int getAmount() {
        return amount;
    }

    public Material getType() {
        return type;
    }

    public List<String> getLore() {
        return lore;
    }

    public int getCustomModelData() {
        return customModelData;
    }

    public Map<Enchantment, Integer> getEnchants() {
        return enchants;
    }

    public Set<ItemFlag> getItemFlags() {
        return itemFlags;
    }

    public List<PotionEffect> getPotionEffects() {
        return potionEffects;
    }

    public List<FireworkEffect> getFireworkEffects() {
        return fireworkEffects;
    }

    public List<Pattern> getPatterns() {
        return patterns;
    }

    public String getSkullTextures() {
        return skullTextures;
    }

    public String getDisplayName() {
        return displayName;
    }

    public short getData() {
        return data;
    }

    public UUID getSkullOwner() {
        return skullOwner;
    }

    public ItemStackBuilder setItemStack(ItemStack item) {
        this.itemStack = item;
        return this;
    }

    public ItemStackBuilder setType(Material type) {
        this.type = type;
        return this;
    }

    public ItemStackBuilder setCustomModelData(int customModelData) {
        this.customModelData = customModelData;
        return this;
    }

    public ItemStackBuilder setAmount(int amount) {
        this.amount = amount;
        return this;
    }

    public ItemStackBuilder setData(int data) {
        if (data > 255) {
            data = 255;
        }
        this.data = (short) data;
        return this;
    }

    public ItemStackBuilder setData(short data) {
        this.data = data;
        return this;
    }

    public ItemStackBuilder setItemMeta(ItemMeta meta) {
        this.meta = meta;
        return this;
    }

    public ItemStackBuilder setDisplayName(String name) {
        this.displayName = name;
        return this;
    }

    public ItemStackBuilder setLore(List<String> loreList) {
        this.lore = loreList;
        return this;
    }

    public ItemStackBuilder setLore(String... lores) {
        this.lore = new ArrayList<String>(Arrays.asList(lores));
        return this;
    }

    public ItemStackBuilder addLore(String lore) {
        this.lore.add(lore);
        return this;
    }

    public ItemStackBuilder addLore(String... lores) {
        this.lore.addAll(Arrays.asList(lores));
        return this;
    }

    public ItemStackBuilder addLore(List<String> lores) {
        for (String lore : lores) {
            this.lore.add(lore);
        }
        return this;
    }

    public ItemStackBuilder addEnchant(Enchantment ench, int level) {
        this.enchants.put(ench, level);
        return this;
    }

    public ItemStackBuilder removeEnchant(Enchantment ench) {
        this.enchants.remove(ench);
        return this;
    }

    public ItemStackBuilder setEnchants(Map<Enchantment, Integer> enchants) {
        this.enchants = enchants;
        return this;
    }

    public ItemStackBuilder setEnchants(List<String> enchants) {
        this.enchants = getEnchantsFromList(enchants);
        return this;
    }

    public ItemStackBuilder setItemFlags(Set<ItemFlag> itemFlags) {
        this.itemFlags = itemFlags;
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag[] itemFlags) {
        this.itemFlags.addAll(Arrays.asList(itemFlags));
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag itemFlag) {
        this.itemFlags.add(itemFlag);
        return this;
    }

    public ItemStackBuilder addPotionEffect(PotionEffect potionEffect) {
        this.potionEffects.add(potionEffect);
        return this;
    }

    public ItemStackBuilder addPattern(Pattern pattern) {
        this.patterns.add(pattern);
        return this;
    }

    public ItemStackBuilder addFireworkEffect(FireworkEffect fireworkEffect) {
        this.fireworkEffects.add(fireworkEffect);
        return this;
    }

    public ItemStackBuilder setSkullTextures(String skullTextures) {
        this.skullTextures = skullTextures;
        return this;
    }

    public ItemStackBuilder setSkullOwner(UUID skullOwner) {
        this.skullOwner = skullOwner;
        return this;
    }

    protected Map<Enchantment, Integer> getEnchantsFromList(List<String> enchants) {
        Map<Enchantment, Integer> map = new HashMap<>();
        if (enchants == null) {
            return map;
        }
        for (String s : enchants) {
            if (s.contains(":")) {
                String[] part = s.split(":");
                Enchantment en = Enchantment.getByName(part[0]);
                if (en == null) {
                    continue;
                }
                map.put(en, Integer.parseInt(part[1]));
            }
        }
        return map;
    }

    protected static ItemStack setEnchants(ItemStack stack, List<String> enchants) {
        if (enchants == null) {
            return stack;
        }
        for (String s : enchants) {
            if (s.contains(":")) {
                String[] part = s.split(":");
                Enchantment en = Enchantment.getByName(part[0]);
                if (en == null) {
                    continue;
                }
                if (stack.getType() != Material.ENCHANTED_BOOK) {
                    stack.addUnsafeEnchantment(en, Integer.parseInt(part[1]));
                } else {
                    EnchantmentStorageMeta esm = (EnchantmentStorageMeta) stack.getItemMeta();
                    esm.addStoredEnchant(en, Integer.parseInt(part[1]), true);
                    stack.setItemMeta((ItemMeta) esm);
                }
            }
        }
        return stack;
    }

    private void setField(Object instance, String name, Object value) throws ReflectiveOperationException {
        Field field = instance.getClass().getDeclaredField(name);
        field.setAccessible(true);
        field.set(instance, value);
    }

}
